# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"create_markdown_table": "00_utils.ipynb",
         "set_up_logging": "00_utils.ipynb",
         "request_access_token": "01_eumetsat.ipynb",
         "query_data_products": "01_eumetsat.ipynb",
         "format_dt_str": "01_eumetsat.ipynb",
         "identify_available_datasets": "01_eumetsat.ipynb",
         "dataset_id_to_link": "01_eumetsat.ipynb",
         "json_extract": "01_eumetsat.ipynb",
         "extract_metadata": "01_eumetsat.ipynb",
         "metadata_maps": "01_eumetsat.ipynb",
         "check_valid_request": "01_eumetsat.ipynb",
         "DownloadManager": "01_eumetsat.ipynb",
         "get_dir_size": "01_eumetsat.ipynb",
         "debug_fp": "01_eumetsat.ipynb",
         "log": "01_eumetsat.ipynb",
         "get_filesize_megabytes": "01_eumetsat.ipynb",
         "eumetsat_filename_to_datetime": "01_eumetsat.ipynb",
         "compress_downloaded_files": "01_eumetsat.ipynb",
         "upload_compressed_files": "01_eumetsat.ipynb",
         "calculate_x_offset": "02_reproj.ipynb",
         "get_seviri_area_def": "02_reproj.ipynb",
         "load_scene": "02_reproj.ipynb",
         "construct_area_def": "02_reproj.ipynb",
         "construct_TM_area_def": "02_reproj.ipynb",
         "reproj_with_manual_grid": "02_reproj.ipynb",
         "reproj_to_xarray": "02_reproj.ipynb",
         "full_scene_pyresample": "02_reproj.ipynb",
         "full_scene_pyinterp": "02_reproj.ipynb",
         "Reprojector": "02_reproj.ipynb",
         "is_retriable": "03_zarr.ipynb",
         "load_from_zarr_bucket": "03_zarr.ipynb",
         "gcsfs.utils.is_retriable": "03_zarr.ipynb",
         "add_constant_coord_to_da": "03_zarr.ipynb",
         "Compressor": "03_zarr.ipynb",
         "save_da_to_zarr": "03_zarr.ipynb",
         "list_blobs_with_prefix": "04_gcp.ipynb",
         "blob_metadata": "04_gcp.ipynb",
         "upload_blob": "04_gcp.ipynb",
         "get_eumetsat_filenames": "04_gcp.ipynb",
         "write_metadata_to_gcp": "04_gcp.ipynb",
         "query": "04_gcp.ipynb",
         "sql_query": "04_gcp.ipynb",
         "project_id": "04_gcp.ipynb",
         "download_eumetsat_files": "05_pipeline.ipynb",
         "download_latest_eumetsat_files": "05_pipeline.ipynb",
         "df_metadata_to_dt_to_fp_map": "05_pipeline.ipynb",
         "reproject_datasets": "05_pipeline.ipynb",
         "compress_and_save_datasets": "05_pipeline.ipynb",
         "download_latest_data_pipeline": "05_pipeline.ipynb",
         "user_key": "05_pipeline.ipynb",
         "user_secret": "05_pipeline.ipynb",
         "slack_webhook_url": "05_pipeline.ipynb",
         "slack_id": "05_pipeline.ipynb"}

modules = ["utils.py",
           "eumetsat.py",
           "reproj.py",
           "io.py",
           "gcp_helpers.py",
           "mario.py"]

doc_url = "https://AyrtonB.github.io/satip/"

git_url = "https://github.com/AyrtonB/satip/tree/master/"

def custom_doc_links(name):
    return None

